package com.ruoyi.bst.device.utils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.domain.enums.DeviceLocationType;
import com.ruoyi.bst.orderDevice.domain.enums.OrderDeviceStatus;
import com.ruoyi.common.utils.MathUtils;
import com.ruoyi.common.utils.map.GpsCoordinateUtils;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.domain.IotDeviceSysInfo;
import com.ruoyi.iot.util.BatUtil;

public class DeviceUtil {

    /**
     * 设置设备系统信息
     * @param device 设备
     * @param sys 设备信息
     */
    public static void setIotSysInfo(DeviceVO device, IotDeviceSysInfo sys, LocalDateTime at, boolean setNull) {
        if (device == null || sys == null) {
            return;
        }

        // 转换经纬度坐标系并赋值
        // 只有定位是正常的才认为是有获取到定位
        if (DeviceUtil.validLocation(sys.getLon(), sys.getLat())) {
            List<BigDecimal> coordinates = GpsCoordinateUtils.coordinateConvert(sys.getLon(), sys.getLat());
            if (coordinates != null && coordinates.size() >= 2) {
                device.setLongitude(coordinates.get(1));
                device.setLatitude(coordinates.get(0));
                device.setLastLocationTime(at);
                device.setLocationType(DeviceLocationType.DEVICE.getCode());
            }
        } else if (setNull) {
            device.setLongitude(null);
            device.setLatitude(null);
            device.setLastLocationTime(null);
            device.setLocationType(null);
        }
        
        device.setLastTime(at);
        device.setVoltage(sys.getBat());
        device.setSignalStrength(sys.getCsq());
        device.setQuality(sys.getQ());
        device.setSatellites(sys.getS());
        device.setIotStatus(sys.getStatus());
        device.setRemainingPower(BatUtil.calcPowerPoint(device.getVoltage(), device.getModelFullVoltage(), device.getModelLowVoltage()));
    }

    /**
     * 设置设备信息
     * @param device 设备
     * @param iot 设备信息
     */
    public static void setIotInfo(DeviceVO device, IotDeviceInfo iot) {
        if (device == null || iot == null) {
            return;
        }

        IotDeviceSysInfo sys = iot.getSys();
        if (sys != null) {
            setIotSysInfo(device, sys, iot.getAt(), false);
        }
        device.setSoftwareVersion(iot.getVer());
    }

    /**
     * 判断设备是否有订单正在使用
     * @param device 设备
     * @return 是否正在使用
     */
    public static boolean isOrderUsing(DeviceVO device) {
        return device != null && device.getOrderDeviceId() != null && OrderDeviceStatus.inUse().contains(device.getOrderDeviceStatus());
    }
    
    /**
     * 判断经纬度是否有效
     * @param lon 经度
     * @param lat 纬度
     * @return 是否有效
     */
    public static boolean validLocation(BigDecimal lon, BigDecimal lat) {
        return lon != null && lat != null 
        && !MathUtils.equals(lon, BigDecimal.ZERO) 
        && !MathUtils.equals(lat, BigDecimal.ZERO);
    }
}